@@ -3,7 +3,7 @@ import { Sequelize } from 'sequelize';
 import pg from 'pg';
 import util from 'util';
 
-import { SYNC_DIRECTIONS, AUDIT_USERID_KEY, SESSION_CONFIG_PREFIX } from '@tamanu/constants';
+import { SYNC_DIRECTIONS, AUDIT_USERID_KEY, SYSTEM_USER_UUID } from '@tamanu/constants';
 import { log } from '@tamanu/shared/services/logging';
 import { serviceContext, serviceName } from '@tamanu/shared/services/logging/context';
 
@@ -11,31 +11,30 @@ import { assertUpToDate, migrate, NON_SYNCING_TABLES } from './migrations';
 import * as models from '../models';
 import { createDateTypes } from './createDateTypes';
 import { setupQuote } from '../utils/pgComposite';
+import { getAuditUserId } from '../utils';
 
 createDateTypes();
 
-export const asyncLocalStorage = new AsyncLocalStorage();
-// this allows us to use transaction callbacks without manually managing a transaction handle
+// This allows us to use transaction callbacks without manually managing a transaction handle
 // https://sequelize.org/master/manual/transactions.html#automatically-pass-transactions-to-all-queries
 // done once for all sequelize objects. Instead of cls-hooked we use the built-in AsyncLocalStorage.
-export const namespace = {
-  bind: () => {}, // compatibility with cls-hooked, not used by sequelize
-  get: (id) => asyncLocalStorage.getStore()?.get(id),
-  set: (id, value) => asyncLocalStorage.getStore()?.set(id, value),
-  run: (callback) => asyncLocalStorage.run(new Map(), callback),
-};
-
-export const setSessionConfigInNamespace = (key, value, callback) => {
-  namespace.run(() => {
-    namespace.set(`${SESSION_CONFIG_PREFIX}${key}`, value);
-    callback()
-  });
-};
-
-export const getSessionConfigInNamespace = (key) => namespace.get(`${SESSION_CONFIG_PREFIX}${key}`);
-
+// NOTE: Do not use this storage for anything, sequelize manages it and may clear it unexpectedly
+const sequelizeTransactionAsyncLocalStorage = new AsyncLocalStorage();
 // eslint-disable-next-line react-hooks/rules-of-hooks
-Sequelize.useCLS(namespace);
+Sequelize.useCLS({
+  bind: () => {}, // compatibility with cls-hooked, not used by sequelize
+  get: id => sequelizeTransactionAsyncLocalStorage.getStore()?.get(id),
+  set: (id, value) => sequelizeTransactionAsyncLocalStorage.getStore()?.set(id, value),
+  run: callback => sequelizeTransactionAsyncLocalStorage.run(new Map(), callback),
+});
+
+// When a transaction is started, Sequelize wraps all calls in a `run()` call on the clsAsyncLocalStorage,
+// so we know we're in a transaction if the value is not empty
+const isInsideTransaction = () => Boolean(sequelizeTransactionAsyncLocalStorage.getStore());
+
+// Once Sequelize has set up the transaction (ie. calling 'START TRANSACTION' and 'SET ISOLATION LEVEL ...') it will add a 'transaction' object to the store
+const isTransactionReady = () =>
+  Boolean(sequelizeTransactionAsyncLocalStorage.getStore()?.get('transaction'));
 
 // this is dangerous and should only be used in test mode
 const unsafeRecreatePgDb = async ({ name, username, password, host, port }) => {
@@ -76,13 +75,16 @@ async function connectToDatabase(dbOptions) {
     alwaysCreateConnection = true,
     loggingOverride = null, // used in tests for migration determinism
     disableChangesAudit = false,
+    recreateDatabase = false,
   } = dbOptions;
   let { name } = dbOptions;
 
   // configure one test db per jest worker
   const workerId = process.env.JEST_WORKER_ID;
-  if (testMode && workerId) {
-    name = `${name}-${workerId}`;
+  if (testMode && (workerId || recreateDatabase)) {
+    if (workerId) {
+      name = `${name}-${workerId}`;
+    }
     if (alwaysCreateConnection) {
       await unsafeRecreatePgDb({ ...dbOptions, name });
     }
@@ -134,17 +136,36 @@ async function connectToDatabase(dbOptions) {
   if (!disableChangesAudit) {
     class QueryWithAuditConfig extends sequelize.dialect.Query {
       async run(sql, options) {
-        const userid = getSessionConfigInNamespace(AUDIT_USERID_KEY);
-        if (userid)
-          await super.run(
-            'SELECT public.set_session_config($1, $2)',
-            [AUDIT_USERID_KEY, userid],
-          );
+        const userid = getAuditUserId();
+        const isInsideATransaction = isInsideTransaction();
+        const isThisTransactionReady = isTransactionReady();
+        // Set audit userid so that any changes in this query are recorded against it
+        // If in a transaction, just use a transaction scoped variable to avoid needing to clear it
+        if (userid) {
+          if (isInsideATransaction && !isThisTransactionReady) {
+            // This may be the 'START TRANSACTION' or 'SET ISOLATION LEVEL ...' queries being run by sequelize.
+            // We don't want to run any queries until the transaction has been set up. So do nothing.
+          } else {
+            await super.run('SELECT public.set_session_config($1, $2, $3)', [
+              AUDIT_USERID_KEY,
+              userid,
+              isInsideATransaction,
+            ]);
+          }
+        }
         try {
           return await super.run(sql, options);
         } catch (error) {
           log.error(error);
           throw error;
+        } finally {
+          // Clear audit userid so that system user changes aren't unintentionally recorded against it
+          if (userid && !isInsideATransaction) {
+            await super.run('SELECT public.set_session_config($1, $2)', [
+              AUDIT_USERID_KEY,
+              SYSTEM_USER_UUID,
+            ]);
+          }
         }
       }
     }
@@ -190,11 +211,11 @@ export async function initDatabase(dbOptions) {
   // of calling it to the implementing server (this allows for skipping migrations
   // in favour of calling sequelize.sync() during test mode)
   // eslint-disable-next-line require-atomic-updates
-  sequelize.migrate = async (direction) => {
+  sequelize.migrate = async direction => {
     await migrate(log, sequelize, direction);
   };
 
-  sequelize.assertUpToDate = async (options) => assertUpToDate(log, sequelize, options);
+  sequelize.assertUpToDate = async options => assertUpToDate(log, sequelize, options);
 
   // init all models
   const modelClasses = Object.values(models);
@@ -205,7 +226,7 @@ export async function initDatabase(dbOptions) {
     primaryKey: true,
   };
   log.info('registeringModels', { count: modelClasses.length });
-  modelClasses.forEach((modelClass) => {
+  modelClasses.forEach(modelClass => {
     if ('initModel' in modelClass) {
       modelClass.initModel(
         {
@@ -222,13 +243,13 @@ export async function initDatabase(dbOptions) {
     }
   });
 
-  modelClasses.forEach((modelClass) => {
+  modelClasses.forEach(modelClass => {
     if (modelClass.initRelations) {
       modelClass.initRelations(models);
     }
   });
 
-  modelClasses.forEach((modelClass) => {
+  modelClasses.forEach(modelClass => {
     if (
       modelClass.syncDirection === SYNC_DIRECTIONS.DO_NOT_SYNC &&
       modelClass.usesPublicSchema &&
@@ -240,8 +261,9 @@ export async function initDatabase(dbOptions) {
     }
   });
 
-  // add isInsideTransaction helper to avoid exposing the asynclocalstorage
-  sequelize.isInsideTransaction = () => !!asyncLocalStorage.getStore();
+  // add isInsideTransaction and isTransactionReady helpers to avoid exposing the asynclocalstorage
+  sequelize.isInsideTransaction = isInsideTransaction;
+  sequelize.isTransactionReady = isTransactionReady;
 
   return { sequelize, models };
 }
